package net.sf.corn.converter;

import java.util.List;

/**
 * Base interface for converter classes.
 * 
 * @author  Serhat Dirik
 *
 * @param <E> Target Transformation Type
 */
public interface IConverter<E> {
	
	/**
	 * Converts java object to targetted type
	 * @param o source java object
	 * @param referenceMap this parameters usually created by highest level transformer to control cyclic dependencies
	 * @return targeted type
	 * @throws ConversionException
	 */
	public E	    	  toTargetType(Object o,ObjectReferenceMap referenceMap) throws ConversionException;
	/**
	 * Converts java object to targetted type
	 * @param o source java object
	 * @return targeted type
	 * @throws ConversionException
	 */
	public E	    	  toTargetType(Object o) throws ConversionException;
	/**
	 * Transforms from target type to Java
	 * @param obj source object
	 * @return java object
	 * @throws ConversionException
	 */
	public Object  	 	  toJava(E obj) throws ConversionException;
	/**
	 * Transforms from target type to Java
	 * @param clazz to be transformed java class
	 * @param obj source object
	 * @return java object 
	 * @throws ConversionException
	 */
	public <T> T  	 	  toJava(Class<T> clazz, E obj) throws ConversionException;
	/**
	 * Gives convertible java types
	 * @return list of convertible classes
	 */
	public List<Class<?>> findConvertibles();
	
	/**
	 * tells if the given class is convertible by the implementation
	 * @param clazz
	 * @return true if convertible
	 */
	public boolean 		  isConvertible(Class<?> clazz);
	/**
	 * tells if the given class is convertible by the implementation
	 * @param className
	 * @return true if convertible
	 */
	public boolean		  isConvertible(String className);

}
