package net.sf.corn.converter.json;

import java.util.AbstractList;
import java.util.AbstractSequentialList;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;
import java.util.Vector;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.management.AttributeList;
import javax.management.relation.RoleList;
import javax.management.relation.RoleUnresolvedList;

import net.sf.corn.converter.ConversionException;
import net.sf.corn.converter.ObjectReferenceMap;
import net.sf.corn.converter.json.JsTypeList.Type;

class CnvList extends AbstractJsonConverter {

	CnvList() {
		getConvertibles().add(List.class);
		getConvertibles().add(ArrayList.class);
		getConvertibles().add(LinkedList.class);
		getConvertibles().add(Vector.class);
		getConvertibles().add(AbstractSequentialList.class);
		getConvertibles().add(AbstractList.class);
		getConvertibles().add(AttributeList.class);
		getConvertibles().add(CopyOnWriteArrayList.class);
		getConvertibles().add(RoleList.class);
		getConvertibles().add(RoleUnresolvedList.class);
		getConvertibles().add(Stack.class);
	}

	@Override
	protected JsTypeObject inToTarget(Object o, ObjectReferenceMap referenceMap)
			throws ConversionException {
		JsTypeList jsonList = new JsTypeList(o.getClass());
		List<?> list = (List<?>) o;
		JsonConverter transformer = new JsonConverter();
		if (referenceMap == null)
			referenceMap = new ObjectReferenceMap();
		for (Object object : list) {
			jsonList.add(transformer.toTargetType(object, referenceMap));
		}
		jsonList.setType(Type.LIST);
		return jsonList;
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	protected Object inToJava(Class<?> clazz, JsTypeObject obj)
			throws ConversionException {
		if (!obj.isList()) {
			throw new ConversionException("Only JsTypeList is allowed");
		}
		JsTypeList jso = (JsTypeList) obj;
		if(jso.getType() != null && ! jso.getType().equals(Type.LIST)){
			throw new ConversionException("Only list typed json list is allowed");
		}
		JsonConverter transformer = new JsonConverter();
		try {
			List al = null;
			try {
				al = (List) clazz.newInstance();
			} catch (InstantiationException e) {
				// private classess or unmodifiable list causes this
				al = new ArrayList<Object>();
			}
			int i = 0;
			for (; i < jso.size(); i++) {
				al.add(transformer.toJava(jso.get(i)));
			}
			return al;
		} catch (Exception e) {
			throw new ConversionException(e);
		}
	}

	@Override
	public boolean isConvertible(Class<?> clazz) {
		boolean ret = checkIfConvertibleFromRegisteredTypes(clazz);
		if (!ret) {
			ret = List.class.isAssignableFrom(clazz);
		}
		return ret;
	}

	@Override
	public boolean isSubjectForCircularReferenceCheck() {
		return true;
	}
}
