package net.sf.corn.converter.json;

import java.lang.reflect.Constructor;

import net.sf.corn.converter.ConversionException;
import net.sf.corn.converter.ObjectReferenceMap;

/**
 * Converter for java classes which has a single String parameter constructor 
 * @author Serhat Dirik
 *
 */
class CnvObject extends AbstractJsonConverter {

	CnvObject() {
	}
 
	protected JsTypeObject inToTarget(Object o, ObjectReferenceMap referenceMap)
			throws ConversionException {
		return new JsTypeSimple(o.toString());
	}

	protected Object inToJava(Class<?> clazz, JsTypeObject obj)
			throws ConversionException {
		if(! obj.isSimple()){
			throw new ConversionException(this.getClass().getName()
					+ " can't be used as converter for " + clazz.getName());
		}
		Object retVal = null;
		try {
			Constructor<?> stringConstructor = clazz.getDeclaredConstructor(String.class);
			retVal = stringConstructor.newInstance(obj.toStringValue());
		} catch (Exception e) {
			throw new ConversionException("An exception occured while transforming json to " + clazz.getName(),e);
		}
		//Reverse must be proven
		if(! obj.toStringValue().equals(retVal.toString())){
			throw new ConversionException(this.getClass().getName()
					+ " can't be used as converter for " + clazz.getName());
		}
		return  retVal;
	}
	
	@Override
	public boolean isConvertible(Class<?> clazz) {
		if(super.isConvertible(clazz)) return true;
		boolean retVal = false;
		try {
			retVal = clazz.getDeclaredConstructor(String.class) != null;
			getConvertibles().add(clazz);
		} catch (Exception e) {
		}
		return retVal;
	}
	
	@Override
	public boolean isSubjectForCircularReferenceCheck() {
		return false;
	}

}
