package net.sf.corn.converter.json;

import java.util.AbstractSet;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.CopyOnWriteArraySet;

import javax.print.attribute.standard.JobStateReasons;

import net.sf.corn.converter.ConversionException;
import net.sf.corn.converter.ObjectReferenceMap;
import net.sf.corn.converter.json.JsTypeList.Type;
/**
 * Converter for {@link java.util.Set} type collections
 * @author Serhat Dirik
 */
class CnvSet extends AbstractJsonConverter {

	CnvSet() {
		getConvertibles().add(Set.class);
		getConvertibles().add(AbstractSet.class);
		getConvertibles().add(CopyOnWriteArraySet.class);
		getConvertibles().add(EnumSet.class);
		getConvertibles().add(HashSet.class);
		getConvertibles().add(JobStateReasons.class);
		getConvertibles().add(LinkedHashSet.class);
		getConvertibles().add(TreeSet.class);
		addIfExist("java.util.concurrent.ConcurrentSkipListSet");
	}

	@Override
	protected JsTypeObject inToTarget(Object o, ObjectReferenceMap referenceMap)
			throws ConversionException {

		JsTypeList jsList = new JsTypeList(o.getClass());
		Set<?> set = (Set<?>) o;
		JsonConverter transformer = new JsonConverter();
		if (referenceMap == null)
			referenceMap = new ObjectReferenceMap();
		for (Object object : set) {
				jsList.add(transformer.toTargetType(object, referenceMap));
		}
		jsList.setType(Type.SET);
		return jsList;
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	protected Object inToJava(Class<?> clazz, JsTypeObject obj)
			throws ConversionException {
		if (!obj.isList()) {
			throw new ConversionException("Only JsTypeList is allowed");
		}
		JsTypeList jso = (JsTypeList) obj;
		if(jso.getType() != null && ! jso.getType().equals(Type.SET)){
			throw new ConversionException("Only set typed json list is allowed");
		}
		try {
			Set al = null;
			try {
				al = (Set) clazz.newInstance();
			} catch (InstantiationException e) {
				al = new HashSet<Object>();
			}

			JsonConverter transformer = new JsonConverter();
			int i = 0;
			for (; i < obj.size(); i++) {
					al.add(transformer.toJava(jso.get(i)));
			}
			return al;
		}catch (IllegalAccessException e) {
			throw new ConversionException(e);
		}
	}

	@Override
	public boolean isConvertible(Class<?> clazz) {
		boolean ret = checkIfConvertibleFromRegisteredTypes(clazz);
		if (!ret) {
			ret = Set.class.isAssignableFrom(clazz);
		}
		return ret;
	}

	@Override
	public boolean isSubjectForCircularReferenceCheck() {
		return true;
	}
}
