package net.sf.corn.converter.json;

import static net.sf.corn.converter.json.IJsonConstants.*;

import java.io.Serializable;

/**
 * The simple json type represents a single object value  
 * @author Serhat Dirik
 */
public class JsTypeSimple extends JsTypeObject implements Serializable {

	private static final long serialVersionUID = -5017949019501994360L;
	private String simpleVal = null;
	
	/**
	 * Default constructor
	 */
	JsTypeSimple() {
	}
	
	/**
	 * Constructs a new object with the given object's java class name and 
	 * it's json converted value
	 * @param className
	 * @param map
	 */
	public JsTypeSimple(Object o) {
		if(o == null){
			throw new IllegalArgumentException("The object value is null");
		}
		setJavaClassName(o.getClass().getName());
		if (o instanceof String || o instanceof Character
				|| o instanceof StringBuilder || o instanceof StringBuffer) {
			setSimpleValue(quote(o.toString()));
		} else {
			setSimpleValue(o.toString());
		}
	}
	/**
	 * Constructs a new simple object 
	 * @param stringVal the value to be kept in the json object
	 * @param javaClassName name of the java class that the value converted from
	 */
	public JsTypeSimple(String stringVal, String javaClassName) {
		setJavaClassName(javaClassName);
		setSimpleValue(stringVal);
	}

	@Override
	public String toJsonString() {

		StringBuilder sb = new StringBuilder();
		if (! isClassHintEnabled() || getJavaClassName() == null) {
			sb.append(getSimpleValue());
		} else {
			sb.append(CHAR_CLASS_START).append(quote(ATTR_CLASSNAME))
					.append(CHAR_FIELD_NAME__VALUE_SEPERATOR)
					.append(quote(getJavaClassName()));
			sb.append(CHAR_FIELD_SEPERATOR)
					.append(quote(ATTR_VALUE))
					.append(CHAR_FIELD_NAME__VALUE_SEPERATOR)
					.append(getSimpleValue() == null ? NULL_STRING_VALUE
							: getSimpleValue());
			sb.append(CHAR_CLASS_END);
		}

		return sb.toString();
	}
	@Override
	public String toString() {
		return toJsonString();

	}
	/**
	 * Gives the contained value
	 * @return the value
	 */
	public String getSimpleValue() {
		return simpleVal;
	}
	/**
	 * Sets the contained value
	 * @param stringVal the value to be set
	 */
	public void setSimpleValue(String stringVal) {
		this.simpleVal = stringVal;
	}
	@Override
	protected Object clone() throws CloneNotSupportedException {
		return new JsTypeSimple(getJavaClassName(), getSimpleValue());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((javaClassName == null) ? 0 : javaClassName.hashCode());
		result = prime * result
				+ ((simpleVal == null) ? 0 : simpleVal.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		JsTypeSimple other = (JsTypeSimple) obj;
		if (javaClassName == null) {
			if (other.javaClassName != null)
				return false;
		} else if (!javaClassName.equals(other.javaClassName))
			return false;
		if (simpleVal == null) {
			if (other.simpleVal != null)
				return false;
		} else if (!simpleVal.equals(other.simpleVal))
			return false;
		return true;
	}

	@Override
	public String toStringValue() {
		String className = getJavaClassName();
		if (className == null
				|| className.equals(String.class.getCanonicalName())
				|| className.equals(Character.class.getCanonicalName())
				|| className.equals(StringBuilder.class.getCanonicalName())
				|| className.equals(StringBuffer.class.getCanonicalName())) {
			return unquote(getSimpleValue());
		}
		return getSimpleValue();
	}

	@Override
	public boolean isNull() {
		return false;
	}

	@Override
	public boolean isComplex() {
		return false;
	}

	@Override
	public boolean isList() {
		return false;
	}

	@Override
	public int size() {
		return getSimpleValue() != null ? 1 : 0;
	}

}
